
#  Copyright 2018 Ocean Protocol Foundation
#  SPDX-License-Identifier: Apache-2.0

KEY_PAIR_MODULUS_BIT = 1024

DID_DDO_CONTEXT_URL = 'https://w3id.org/did/v1'

PROOF_TYPE = 'DDOIntegritySignature'
